# This file is @generated by tarina.lang CLI tool
# It is not intended for manual editing.

from tarina.lang.model import LangItem, LangModel


class Lang_Error:
    locale: LangItem = LangItem("lang", "error.locale")
    scope: LangItem = LangItem("lang", "error.scope")
    type: LangItem = LangItem("lang", "error.type")


class Lang_:
    error = Lang_Error
    miss_require_scope: LangItem = LangItem("lang", "miss_require_scope")
    miss_require_type: LangItem = LangItem("lang", "miss_require_type")


class Lang(LangModel):
    lang = Lang_



from __future__ import annotations

import functools
import inspect
from collections.abc import Mapping
from typing import Any, Callable


@functools.lru_cache(4096)
def get_signature(target: Callable):
    return inspect.signature(target).parameters.values()


try:
    from inspect import get_annotations  # type: ignore
except ImportError:

    def get_annotations(
        obj: Callable,
        *,
        _globals: Mapping[str, Any] | None = None,
        _locals: Mapping[str, Any] | None = None,
        eval_str: bool = False,
    ) -> dict[str, Any]:  # sourcery skip: avoid-builtin-shadow
        if not callable(obj):
            raise TypeError(f"{obj!r} is not a module, class, or callable.")

        ann = getattr(obj, "__annotations__", None)
        obj_globals = getattr(obj, "__globals__", None)
        obj_locals = None
        unwrap = obj
        if ann is None:
            return {}

        if not isinstance(ann, dict):
            raise ValueError(f"{unwrap!r}.__annotations__ is neither a dict nor None")
        if not ann:
            return {}

        if not eval_str:
            return dict(ann)

        if unwrap is not None:
            while True:
                if hasattr(unwrap, "__wrapped__"):
                    unwrap = unwrap.__wrapped__  # type: ignore
                    continue
                if isinstance(unwrap, functools.partial):
                    unwrap = unwrap.func
                    continue
                break
            if hasattr(unwrap, "__globals__"):
                obj_globals = unwrap.__globals__

        if _globals is None:
            _globals = obj_globals
        if _locals is None:
            _locals = obj_locals

        return {
            key: eval(value, _globals, _locals) if isinstance(value, str) else value  # type: ignore
            for key, value in ann.items()
        }


@functools.lru_cache(4096)
def signatures(callable_target: Callable) -> list[tuple[str, Any, Any]]:
    callable_annotation = get_annotations(callable_target, eval_str=True)
    return [
        (
            param.name,
            (
                (callable_annotation.get(param.name) if isinstance(param.annotation, str) else param.annotation)
                if param.annotation is not inspect.Signature.empty
                else None
            ),
            param.default,
        )
        for param in get_signature(callable_target)
    ]



CRLF: str
QUOTATION: dict[str, str]

def split(text: str, separator: str, crlf: bool = True) -> list[str]:
    """尊重引号与转义的字符串切分

    Args:
        text (str): 要切割的字符串
        separator (str): 切割符.
        crlf (bool): 是否去除 \n 与 \r，默认为 True

    Returns:
        List[str]: 切割后的字符串, 可能含有空格
    """
    ...

def split_once(text: str, separator: str, crlf: bool = True) -> tuple[str, str]:
    """尊重引号与转义的字符串切分, 只切割一次

    Args:
        text (str): 要切割的字符串
        separator (str): 切割符.
        crlf (bool): 是否去除 \n 与 \r，默认为 True
    Returns:
        Tuple[str, str]: 切割后的字符串, 可能含有空格
    """
    ...

def split_once_without_escape(text: str, separator: str, crlf: bool = True) -> tuple[str, str]:
    """尊重引号的字符串切分, 只切割一次

    Args:
        text (str): 要切割的字符串
        separator (str): 切割符.
        crlf (bool): 是否去除 \n 与 \r，默认为 True
    Returns:
        Tuple[str, str]: 切割后的字符串, 可能含有空格
    """
    ...

def split_once_index_only(text: str, separator: str, offset: int, crlf: bool = True):
    """尊重引号的字符串切分, 只切割一次

    Args:
        text (str): 要切割的字符串
        separator (str): 切割符.
        offset (int): 起始位置
        crlf (bool): 是否去除 \n 与 \r，默认为 True

    Returns:
        Tuple[str, str]: 切割后的字符串, 可能含有空格
    """
    ...


class String:
    left_index: int
    right_index: int
    next_index: int
    _len: int
    text: str

    def __init__(self, text: str): ...

    def step(self, separator: str, crlf: bool = True) -> None: ...

    def val(self) -> str: ...

    def apply(self) -> None: ...

    @property
    def complete(self) -> bool: ...



from __future__ import annotations

QUOTATION = {"'": "'", '"': '"'}
CRLF = "\n\r"


def split_once(text: str, separator: str, crlf: bool = True):
    """尊重引号与转义的字符串切分, 只切割一次

    Args:
        text (str): 要切割的字符串
        separator (str): 切割符.
        crlf (bool): 是否去除 \n 与 \r，默认为 True

    Returns:
        Tuple[str, str]: 切割后的字符串, 可能含有空格
    """
    if crlf:
        separator += CRLF
    index, out_text, quotation, escape, sep = 0, "", "", False, False
    text = text.lstrip()
    first_quoted_sep_index = -1
    last_quote_index = 0
    tlen = len(text)
    for char in text:
        index += 1
        if char in separator:
            if not quotation:
                sep = True
                continue
            if first_quoted_sep_index == -1:
                first_quoted_sep_index = index
        if sep:
            index -= 1
            break
        if char in QUOTATION:  # 遇到引号括起来的部分跳过分隔
            if index == 1 + escape + last_quote_index and not quotation:
                quotation = QUOTATION[char]
            elif text[index - 2] not in separator and char == quotation:
                last_quote_index = index
                quotation = ""
                first_quoted_sep_index = -1
            else:
                out_text += char
            if escape:
                out_text = out_text[:-1] + char
        else:
            out_text += char
        escape = char == "\\"
    if index == tlen:
        if first_quoted_sep_index == -1:
            return out_text if last_quote_index else text.rstrip(separator), ""
        return text[: first_quoted_sep_index - 1], text[first_quoted_sep_index:]
    return out_text, text[index:]


def split_once_without_escape(text: str, separator: str, crlf: bool = True):
    """尊重引号的字符串切分, 只切割一次

    Args:
        text (str): 要切割的字符串
        separator (str): 切割符.
        crlf (bool): 是否去除 \n 与 \r，默认为 True

    Returns:
        Tuple[str, str]: 切割后的字符串, 可能含有空格
    """
    if crlf:
        separator += CRLF
    index, quotation = 0, ""
    text = text.lstrip()
    first_quoted_sep_index = -1
    last_quote_index = 0
    tlen = len(text)
    for char in text:
        index += 1
        if char in separator:
            if not quotation:
                break
            if first_quoted_sep_index == -1:
                first_quoted_sep_index = index
        if char in QUOTATION:  # 遇到引号括起来的部分跳过分隔
            if index == 1 + last_quote_index and not quotation:
                quotation = QUOTATION[char]
            elif text[index - 2] not in separator and char == quotation:
                last_quote_index = index
                quotation = ""
                first_quoted_sep_index = -1
    if index == tlen:
        if first_quoted_sep_index == -1:
            return text[:last_quote_index] if last_quote_index else text.rstrip(separator), ""
        return text[: first_quoted_sep_index - 1], text[first_quoted_sep_index:]
    return text[:index-1], text[index:]


def split_once_index_only(text: str, separator: str, offset: int, crlf: bool = True):
    """尊重引号的字符串切分, 只切割一次

    Args:
        text (str): 要切割的字符串
        separator (str): 切割符.
        offset (int): 起始位置
        crlf (bool): 是否去除 \n 与 \r，默认为 True

    Returns:
        Tuple[str, str]: 切割后的字符串, 可能含有空格
    """
    if crlf:
        separator += CRLF
    index = offset
    quotation = ""
    sep = 0
    text = text[offset:]
    first_quoted_sep_index = -1
    last_quote_index = 0
    tlen = len(text)
    for char in text:
        index += 1
        if char in separator:
            if not quotation:
                sep += 1
                continue
            if first_quoted_sep_index == -1:
                first_quoted_sep_index = index
        if sep:
            index -= 1
            break
        if char in QUOTATION:  # 遇到引号括起来的部分跳过分隔
            if index == 1 + last_quote_index and not quotation:
                quotation = QUOTATION[char]
            elif text[index - offset- 2] not in separator and char == quotation:
                last_quote_index = index
                quotation = ""
                first_quoted_sep_index = -1
    if index == tlen:
        if first_quoted_sep_index == -1:
            return index, sep
        return first_quoted_sep_index, sep
    return index, sep


def split(text: str, separator: str, crlf: bool = True):
    """尊重引号与转义的字符串切分

    Args:
        text (str): 要切割的字符串
        separator (str): 切割符.
        crlf (bool): 是否去除 \n 与 \r，默认为 True

    Returns:
        List[str]: 切割后的字符串, 可能含有空格
    """
    if crlf:
        separator += CRLF
    text = text.strip(separator)
    result, quotation, escape = [], "", False
    quoted_sep_index = []
    last_sep_index = 0
    last_quote_index = 0
    index = 0
    for char in text:
        index += 1
        if char in QUOTATION:
            if index == 1 + escape + max(last_sep_index, last_quote_index) and not quotation:
                quotation = QUOTATION[char]
            elif (not result or result[-1] not in separator) and char == quotation:
                quotation = ""
                last_quote_index = index
            else:
                result.append(char)
            if escape:
                result[-1] = char
        elif char in separator:
            if quotation:
                quoted_sep_index.append(len(result) + 1)
                result.append(char)
            else:
                last_sep_index = index
                if result and result[-1] != "\0":
                    result.append("\0")
        else:
            result.append(char)
        escape = char == "\\"
    if not result:
        return []
    if quotation and quoted_sep_index:
        result.insert(last_sep_index, text[last_quote_index or last_sep_index])
        for i in quoted_sep_index:
            result[i] = "\0"
    return str.join("", result).split("\0")


class String:
    left_index: int
    right_index: int
    next_index: int
    _len: int
    text: str

    def __init__(self, text: str):
        self.text = text
        self._len = len(text)
        self.left_index = 0
        self.right_index = 0
        self.next_index = 0

    def step(self, separator: str, crlf: bool = True):
        self.next_index, offset = split_once_index_only(self.text, separator, self.left_index, crlf)
        self.right_index = self.next_index - offset

    def val(self):
        return self.text[self.left_index:self.right_index]

    def apply(self):
        self.left_index = self.next_index
        self.right_index = self._len

    @property
    def complete(self):
        return self.left_index == self._len

    def __repr__(self):
        return f"String({self.text!r}[{self.left_index}:{self.right_index}])"

    def __str__(self):
        return self.val()



from __future__ import annotations

from contextlib import contextmanager
from contextvars import ContextVar, Token
from typing import Generic, TypeVar

T = TypeVar("T")
D = TypeVar("D")


class ContextModel(Generic[T]):
    current_ctx: ContextVar[T]

    def __init__(self, name: str) -> None:
        self.current_ctx = ContextVar(name)

    def get(self, default: T | D | None = None) -> T | D:
        return self.current_ctx.get(default)  # type: ignore

    def set(self, value: T):
        return self.current_ctx.set(value)

    def reset(self, token: Token):
        return self.current_ctx.reset(token)

    @contextmanager
    def use(self, value: T):
        token = self.set(value)
        yield
        self.reset(token)



def test_generic_isinstance():
    from typing import Any, Dict, List, Literal, TypeVar, Union, TypedDict

    from typing_extensions import Annotated

    from tarina import generic_isinstance

    S = TypeVar("S", bound=str)
    assert generic_isinstance("a", Any)
    assert generic_isinstance(1, int)
    assert generic_isinstance(1, Union[str, int])
    assert generic_isinstance(123, Literal[123])
    assert generic_isinstance([1], List[int])
    assert generic_isinstance([1], list[int])
    assert generic_isinstance(1, Annotated[int, lambda x: x > 0])
    assert generic_isinstance("a", S)
    assert generic_isinstance(1, (int, str))
    assert generic_isinstance("a", (int, str))
    assert generic_isinstance((1, "a"), tuple[int, str])
    assert generic_isinstance((1, 2, 3), tuple[int, ...])
    assert not generic_isinstance(bool, (str, list))
    assert not generic_isinstance({123}, Dict[str, str])
    assert not generic_isinstance([1], List[str])
    assert not generic_isinstance({"a": 1}, dict[str, str])
    assert not generic_isinstance((1, 2), tuple[int, str])
    assert not generic_isinstance((1, "a"), tuple[int, str, str])

    class User(TypedDict):
        name: str
        age: int

    assert generic_isinstance({"name": "test", "age": 1}, User)
    assert not generic_isinstance({"name": "test", "age": "1"}, User)


def test_generic_issubclass():
    from typing import Any, Dict, List, Literal, TypeVar, Union

    from typing_extensions import Annotated

    from tarina import generic_issubclass

    S = TypeVar("S", bound=str)
    assert generic_issubclass(str, Any)
    assert generic_issubclass(int, Union[str, int])
    assert generic_issubclass(List[int], List[int])
    assert generic_issubclass(int, Annotated[int, lambda x: x > 0])
    assert generic_issubclass(str, S)


def test_lru():
    """测试 LRU缓存"""
    from tarina import LRU

    cache: LRU[str, str] = LRU(3)
    cache["a"] = "a"
    cache["b"] = "b"
    cache["c"] = "c"
    assert cache.peek_first_item()[1] == "c"  # type: ignore
    _ = cache.get("a", Ellipsis)
    print(f"\n{cache.items()}")
    assert cache.peek_first_item()[1] == "a"  # type: ignore
    cache["d"] = "d"
    assert cache.get("b", Ellipsis) == Ellipsis


def test_split_once():
    """测试单次分割函数, 能以引号扩起空格, 并允许保留引号"""
    from tarina import split_once

    assert split_once("arclet-alconna", " ") == ("arclet-alconna", "")
    text1 = "rrr b bbbb"
    text2 = "'rrr b' bbbb"
    text3 = "\\'rrr b\\' bbbb"
    text4 = "\\'rrr \\b\\' bbbb"
    assert split_once(text1, " ") == ("rrr", "b bbbb")
    assert split_once(text2, " ") == ("rrr b", "bbbb")
    assert split_once(text3, " ") == ("'rrr b'", "bbbb")
    assert split_once(text4, " ") == ("'rrr \\b'", "bbbb")  # 不消除其他转义字符斜杠
    assert split_once('\\"\\b"""""""""""', " ") == ('"\\b', "")
    assert split_once('\\"\\b""""\\"""""""', " ") == ('"\\b"', "")
    assert split_once('\\"\\b""""""""""\\"', " ") == ('"\\b"', "")
    assert split_once("'rrr b\" b' bbbb", " ") == ('rrr b" b', "bbbb")
    assert split_once("rrr 'bbb'", " ") == ("rrr", "'bbb'")
    assert split_once("'rrr bbb'", " ") == ("rrr bbb", "")

    assert split_once(
        r'\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\",\\"',
        " ",
    ) == (
        r'",\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\"\\",\\",\\"',
        "",
    )

    assert (split_once("123456789", " ", True)) == ("123456789", "")
    assert (split_once("123 456 789", " ", True)) == ("123", "456 789")
    assert (split_once('"123 456" 789', " ", True)) == ("123 456", "789")
    assert (split_once('12"3 456" 789', " ", True)) == ('12"3', '456" 789')
    assert (split_once('12"3 456 789', " ", True)) == ('12"3', "456 789")
    assert (split_once('"123 456 789', " ", True)) == ('"123', "456 789")
    assert (split_once('123" 456 789', " ", True)) == ('123"', "456 789")
    assert (split_once('"""123 456"" 789', " ", True)) == ('"""123 456""', "789")
    assert (split_once('"123 "456 789', " ", True)) == ('"123', '"456 789')

    assert split_once("123 ", " ") == ("123", "")
    assert split_once("123  ", " ") == ("123", "")
    assert split_once("123   456", " ") == ("123", "456")


def test_split():
    """测试分割函数, 能以引号扩起空格, 并允许保留引号"""
    from tarina import split

    text1 = "rrr b bbbb"
    text2 = "'rrr b' bbbb"
    text3 = "\\'rrr b\\' bbbb"
    assert split(text1, " ") == ["rrr", "b", "bbbb"]
    assert split(text2, " ") == ["rrr b", "bbbb"]
    assert split(text3, " ") == ["'rrr b'", "bbbb"]
    assert split("", " ") == []
    assert split("  ", " ") == []

    assert (split("123 456 789", " ", True)) == ["123", "456", "789"]
    assert (split('123 "456 789" abc', " ", True)) == ["123", "456 789", "abc"]
    assert (split('123 45"6 789 abc', " ", True)) == ["123", '45"6', "789", "abc"]
    assert (split('123 456" 789 abc', " ", True)) == ["123", '456"', "789", "abc"]
    assert (split('123 456 "789 abc def', " ", True)) == ["123", "456", '"789', "abc", "def"]
    assert (split('123 "456 "789', " ", True)) == ["123", '"456', '"789']
    assert (split('123 """456 7" 789', " ", True)) == ["123", "456 7", "789"]
    assert (split('123 """456 "789', " ", True)) == ["123", '"456', '"789']

    assert (split("123 456 ", " ")) == ["123", "456"]
    assert (split("123 456  ", " ")) == ["123", "456"]
    assert (split("123   456", " ")) == ["123", "456"]


def test_string():
    from tarina import String

    s = String("123 456 789")
    s.step(" ")
    assert s.val() == "123"
    s.apply()
    s.step(" ")
    assert s.val() == "456"
    assert s.val() == "456"
    # only apply can change the value
    s.step(" ")
    assert s.val() == "456"
    assert not s.complete
    s.apply()
    s.step(" ")
    assert s.val() == "789"
    s.apply()
    assert s.complete

def test_lang():
    """测试 i18n"""
    from pathlib import Path

    from tarina import lang

    assert lang.locales == {"zh-CN", "en-US"}
    lang.select("zh-CN")
    assert lang.current == "zh-CN"
    assert lang.require("lang", "error.type") == "'{target}' 在 '{locale}:{scope}' 不是合法的类型"
    assert lang.require("lang", "error.type", "en-US") == "'{target}' is not a valid type in '{locale}:{scope}'"
    lang.select("en-US")
    assert lang.current == "en-US"
    try:
        lang.select("ru-RU")
    except ValueError as e:
        assert str(e) == "'ru-RU' is not a valid language locale"
    try:
        lang.load_data("test", {})
    except KeyError as e:
        assert str(e) == "\"lang file 'test' missed require scope 'lang'\""
    lang.load_data("test", {"lang": {"error": {"type": "test"}}})
    lang.select("test")
    assert lang.require("lang", "error.type") == "test"
    assert lang.require("lang", "error.locale") == "'{target}' 不是合法的语种"
    scope_lang = lang.dispatch("lang")
    assert scope_lang.require("error.type") == "test"
    assert scope_lang.require("error.locale") == "'{target}' 不是合法的语种"
    lang.load_file(Path(__file__).parent / "en-UK.yml")
    assert lang.locales == {"zh-CN", "test", "en-US", "en-UK"}


def test_init_spec():
    from dataclasses import dataclass

    from tarina import init_spec

    @dataclass
    class User:
        name: str
        age: int

    @init_spec(User)
    def add_user(user: User):
        return user

    assert add_user("test", age=1) == User("test", 1)


def test_date_parser():
    from datetime import datetime, timedelta

    from tarina import DateParser

    assert DateParser.parse("2021-01-01") == datetime(2021, 1, 1)
    assert DateParser.parse("2021-01-01 12:34:56") == datetime(2021, 1, 1, 12, 34, 56)
    assert DateParser.parse("1m").replace(microsecond=0) == datetime.now().replace(microsecond=0) + timedelta(minutes=1)
    assert DateParser.parse("h3").replace(microsecond=0) == datetime.now().replace(microsecond=0) + timedelta(hours=3)


def test_safe_eval():
    from tarina import safe_eval

    ctx = {"a": {"b": ["c"]}}

    assert safe_eval("a['b'][0]", ctx) == "c"

    class A:
        class B:
            c = 123

    assert safe_eval("A.B.c", {"A": A}) == 123



from __future__ import annotations

import contextlib
import inspect
import json
import locale
import os
import sys
from dataclasses import dataclass, field
from pathlib import Path
from typing import Callable, Final, Union, final

from typing_extensions import Self

root_dir: Final[Path] = Path(__file__).parent.parent / "i18n"
WINDOWS = sys.platform.startswith("win") or (sys.platform == "cli" and os.name == "nt")


@dataclass
class _LangConfigData:
    default: str | None = None
    frozen: dict[str, list[str]] | None = None
    require: dict[str, list[str]] | None = None
    name: str | None = None

    locales: set[str] = field(default_factory=set)

    def __post_init__(self):
        if isinstance(self.name, str):
            self.name = self.name.strip()
            if not self.name:
                self.name = None


def _get_win_locale_with_ctypes() -> str | None:
    import ctypes

    kernel32 = ctypes.windll.kernel32
    lcid: int = kernel32.GetUserDefaultUILanguage()
    return locale.windows_locale.get(lcid)


def _get_win_locale_from_registry() -> str | None:
    import winreg  # noqa

    with contextlib.suppress(Exception):
        with winreg.OpenKey(winreg.HKEY_CURRENT_USER, r"Control Panel\International") as key:
            if lcid := winreg.QueryValueEx(key, "Locale")[0]:
                return locale.windows_locale.get(int(lcid, 16))


if WINDOWS:
    try:
        import ctypes  # noqa: F401

        _get_win_locale = _get_win_locale_with_ctypes
    except ImportError:
        _get_win_locale = _get_win_locale_from_registry


def get_locale() -> str | None:
    if WINDOWS:
        return _get_win_locale()

    return locale.getlocale(locale.LC_MESSAGES)[0]  # type: ignore


def _get_config(root: Path) -> _LangConfigData:
    if not (root / ".config.json").exists():
        raise FileNotFoundError(f"Config file not found in {root}")
    with (root / ".config.json").open("r", encoding="utf-8") as f:
        data = json.load(f)
        if "frozen" in data:
            data["frozen"] = _expand(data["frozen"])
        if "require" in data:
            data["require"] = _expand(data["require"])
        return _LangConfigData(**data)


Types = Union[str, dict[str, "Types"]]
Raw = dict[str, dict[str, Types]]


def convert_dictionary(data, prefix: str = ""):
    result = {}
    for key, value in data.items():
        if isinstance(value, dict):
            result.update(convert_dictionary(value, f"{prefix}{key}."))
        else:
            result[f"{prefix}{key}"] = value
    return result


def flatten(data: Raw) -> dict[str, dict[str, str]]:
    result = {}
    for scope, types in data.items():
        result[scope] = {}
        for type, value in types.items():
            if isinstance(value, dict):
                result[scope].update(convert_dictionary({type: value}))
            else:
                result[scope][type] = value
    return result


def merge(source: dict, target: dict) -> dict:
    for key, value in source.items():
        if isinstance(value, dict):
            target[key] = merge(value, target.get(key, {}))
        elif isinstance(value, list):
            target[key] = value + target.get(key, [])
        else:
            target[key] = value
    return target


def _expand(data: list[str]):
    result: dict[str, list[str]] = {}
    for i in data:
        if "." not in i:
            i += ".*"
        parts = i.split(".", 1)
        s = parts[0]
        if s not in result:
            result[s] = []
            if not parts[1].startswith("*"):
                result[s].append(parts[1])
        elif result[s] and not parts[1].startswith("*"):
            result[s].append(parts[1])
    return result


def _get_lang(file: Path) -> Raw:
    if not file.exists():
        raise FileNotFoundError(f"Lang file '{file}' not found")
    if file.suffix.startswith(".json"):
        with file.open("r", encoding="utf-8") as f:
            data: Raw = json.load(f)
        data.pop("$schema", None)
        return data
    try:
        import yaml
    except ImportError:
        raise ImportError("PyYAML is required to load yaml file")
    with file.open("r", encoding="utf-8") as f:
        data: Raw = yaml.safe_load(f)
    data.pop("$schema", None)
    return data


def _get_scopes(root: Path) -> dict[str, dict[str, dict[str, str]]]:
    result = {}
    for i in root.iterdir():
        if not i.is_file() or i.name.startswith(".") or i.suffix not in (".json", ".yaml", ".yml"):
            continue
        result[i.stem] = flatten(_get_lang(i))
    return result


class _LangScope:
    def __init__(self, scope: str):
        self.scope = scope

    def require(self, type: str, subtype: str | None = None, locale: str | None = None):
        if subtype:
            type = f"{type}.{subtype}"
        return lang.require(self.scope, type, locale)

    def set(self, type: str, content: str, locale: str | None = None):
        return lang.set(self.scope, type, content, locale)


@final
class _LangConfig:
    def __init__(self):
        self._root_config = _get_config(root_dir)
        self.__configs: dict[str, _LangConfigData] = {"$root": self._root_config}
        self.__locale: str = self._root_config.default or "en-US"
        self.__default_locale: str = self._root_config.default or "en-US"
        self.__langs: dict[str, dict[str, dict[str, str]]] = _get_scopes(root_dir)
        self.__locales = set(self.__langs.keys())
        self.__frozen: dict[str, list[str]] = (self._root_config.frozen or {}).copy()
        self.callbacks: list[Callable[[str], None]] = []
        self.select_local()

    @property
    def locales(self):
        return set(sorted(self.__locales))  # noqa: C414

    def locales_in(self, config_name: str):
        return self.__configs[config_name].locales

    @property
    def current(self):
        return self.__locale

    def select_local(self):
        """
        依据系统语言尝试自动选择语言
        """
        old = self.__locale
        if (lc := get_locale()) and lc.replace("_", "-") in self.__langs:
            self.__locale = lc.replace("_", "-")
            if old != self.__locale:
                for i in self.callbacks:
                    i(self.__locale)
        return self

    def select(self, locale: str) -> Self:
        old = self.__locale
        locale = locale.replace("_", "-")
        if locale not in self.__langs:
            raise ValueError(self.require("lang", "error.locale").format(target=locale))
        self.__locale = locale
        if old != self.__locale:
            for i in self.callbacks:
                i(self.__locale)
        return self

    def load_data(self, locale: str, data: Raw, config: _LangConfigData | None = None):
        if config:
            config.locales.add(locale)
        if locale in self.__langs:
            source = flatten(data)
            target = self.__langs[locale]
            for scope, ignores in self.__frozen.items():
                if scope not in target or scope not in source:
                    continue
                if not ignores:
                    source.pop(scope)
                else:
                    for i in ignores:
                        source[scope] = {
                            k: v for k, v in source[scope].items() if not (k.startswith(i) and k in target[scope])
                        }
            self.__langs[locale] = merge(source, target)
        else:
            self.__locales.add(locale)
            self.__langs[locale] = flatten(data)
        if not config or not config.require:
            return
        for scope, requries in config.require.items():
            if scope not in self.__langs[locale]:
                raise KeyError(self.require("lang", "miss_require_scope", locale).format(locale=locale, target=scope))
            for t in requries:
                if any(k.startswith(t) for k in self.__langs[locale][scope]):
                    continue
                raise KeyError(
                    self.require("lang", "miss_require_type", locale).format(locale=locale, scope=scope, target=t)
                )

    def load_file(self, filepath: Path, config: _LangConfigData | None = None):
        return self.load_data(filepath.stem, _get_lang(filepath), config)

    def load_config(self, dir_path: Path, config_name: str | None = None):
        config = _get_config(dir_path)
        name = config.name or config_name
        if not name:
            raise ValueError("Config name is required")
        if name.startswith("$"):
            raise ValueError("Config name cannot start with '$'")
        self.__default_locale = config.default or self.__default_locale
        self.__configs[name] = config
        self.__frozen = merge(config.frozen or {}, self.__frozen)
        self.select_local()
        return config

    def load(self, root: Path) -> Self:
        if (cf := inspect.currentframe()) and (fb := cf.f_back):
            mod_name = fb.f_globals["__loader__"].name
            mod_name = mod_name.removesuffix(".i18n")
        else:
            mod_name = None
        config = self.load_config(root, mod_name)
        for i in root.iterdir():
            if not i.is_file() or i.name.startswith(".") or i.suffix not in (".json", ".yaml", ".yml"):
                continue
            self.load_file(i, config)
        return self

    def require(self, scope: str, type: str, locale: str | None = None) -> str:
        locale = locale or self.__locale
        if locale not in self.__langs:
            raise ValueError(self.__langs[self.__locale]["lang"]["error.locale"].format(target=locale))
        if scope in self.__langs[locale]:
            _types = self.__langs[locale][scope]
        elif scope in self.__langs[self.__locale]:
            _types = self.__langs[self.__locale][scope]
        elif scope in self.__langs[self.__default_locale]:
            _types = self.__langs[self.__default_locale][scope]
        else:
            raise ValueError(self.__langs[locale]["lang"]["error.scope"].format(target=scope, locale=locale))
        if type in _types:
            return _types[type]
        elif type in self.__langs[self.__locale][scope]:
            return self.__langs[self.__locale][scope][type]
        elif type in self.__langs[self.__default_locale][scope]:
            return self.__langs[self.__default_locale][scope][type]
        else:
            raise ValueError(self.__langs[locale]["lang"]["error.type"].format(target=type, locale=locale, scope=scope))

    def set(self, scope: str, type: str, content: str, locale: str | None = None):
        locale = locale or self.__locale
        if locale not in self.__langs:
            raise ValueError(self.__langs[self.__locale]["lang"]["error.locale"].format(target=locale))
        if scope in self.__frozen:
            raise ValueError(self.__langs[locale]["lang"]["error.scope"].format(target=scope, locale=locale))
        elif type in self.__frozen.get(scope, []):
            raise ValueError(self.__langs[locale]["lang"]["error.type"].format(target=type, locale=locale, scope=scope))
        self.__langs[locale].setdefault(scope, {})[type] = content

    def dispatch(self, scope: str) -> _LangScope:
        return _LangScope(scope)

    def __repr__(self):
        return f"<LangConfig: {self.__locale}>"


lang: _LangConfig = _LangConfig()

__all__ = ["lang"]



from __future__ import annotations

import sys
import types
from collections.abc import Iterable, Mapping
from itertools import repeat
from types import GenericAlias
from typing import Annotated, Any, Literal, TypeVar, Union, TypedDict

from typing_extensions import Literal as typing_ext_Literal
from typing_extensions import get_args
from typing_extensions import get_origin as typing_ext_get_origin

Unions = (Union, types.UnionType) if sys.version_info >= (3, 10) else (Union,)  # pragma: no cover
_TypedDictMeta = type(TypedDict.__mro_entries__(...)[0])  # type: ignore


def origin_is_union(origin: type | None) -> bool:
    return any(origin is u for u in Unions)


def origin_is_literal(origin: type | None) -> bool:
    """判断是否是 Literal 类型"""
    return origin is Literal or origin is typing_ext_Literal


def get_origin(obj: Any) -> Any:
    return typing_ext_get_origin(obj) or obj


def isclass(cls: Any) -> bool:
    return isinstance(cls, type) and not isinstance(cls, GenericAlias)


def generic_isinstance(obj: Any, par: Any) -> bool:
    """检查 obj 是否是 args 中的一个类型, 支持泛型, Any, Union
    Args:
        obj (Any): 要检查的对象
        par (Any): 要检查的对象的类型
    Returns:
        bool: 是否是类型
    """
    if par is Any:
        return True
    _origin = get_origin(par)
    try:
        if _origin is Annotated:
            return generic_isinstance(obj, get_args(par)[0])
        if _origin is Literal:
            return obj in get_args(par)
        if _origin in Unions:  # pragma: no cover
            for p in get_args(par):
                if generic_isinstance(obj, p):
                    return True
        if par.__class__ is tuple:
            for p in par:
                if generic_isinstance(obj, p):
                    return True
        if par.__class__ is _TypedDictMeta:
            if not isinstance(obj, Mapping):
                return False
            for k, v in par.__annotations__.items():
                if k in par.__required_keys__:
                    if k not in obj:
                        return False
                    if not generic_isinstance(obj[k], v):
                        return False
                elif k in par.__optional_keys__:
                    if k in obj and not generic_isinstance(obj[k], v):
                        return False
            return True
        if isclass(par):
            return isinstance(obj, par)
        if isinstance(par, TypeVar):  # pragma: no cover
            if par.__constraints__:
                return any(generic_isinstance(obj, p) for p in par.__constraints__)
            return generic_isinstance(obj, par.__bound__) if par.__bound__ else True
        if isinstance(obj, _origin):  # type: ignore
            if args := get_args(par):
                if _origin is tuple:
                    if len(args) == 2 and args[1] is Ellipsis:
                        return all(map(generic_isinstance, obj, repeat(args[0])))
                    return len(args) == len(obj) and all(map(generic_isinstance, obj, args))
                elif len(args) == 1 and isinstance(obj, Iterable):
                    return all(map(generic_isinstance, obj, repeat(args[0])))
                elif len(args) == 2 and isinstance(obj, Mapping):
                    return all(map(generic_isinstance, obj.keys(), repeat(args[0]))) and all(
                        map(generic_isinstance, obj.values(), repeat(args[1]))
                    )
            return True
    except TypeError:
        return False
    return False


def generic_issubclass(scls: Any, cls: Any) -> Any:
    """检查 scls 是否是 cls 中的一个子类, 支持泛型, Any, Union
    Args:
        scls (Any): 要检查的类
        cls (Any): 要检查的类的父类
    Returns:
        bool: 是否是父类
    """
    if cls is Any:
        return True

    if scls is Any:
        return cls

    if isclass(scls) and (isclass(cls) or isinstance(cls, tuple)):
        return issubclass(scls, cls)

    scls_origin, scls_args = get_origin(scls) or scls, get_args(scls)
    cls_origin, cls_args = get_origin(cls) or cls, get_args(cls)

    if scls_origin is tuple and cls_origin is tuple:
        if len(scls_args) == 2 and scls_args[1] is Ellipsis:
            return generic_issubclass(scls_args[0], cls_args)

        if len(cls_args) == 2 and cls_args[1] is Ellipsis:
            return all(map(generic_issubclass, scls_args, repeat(cls_args[0])))

    if scls_origin is Annotated:
        return generic_issubclass(scls_args[0], cls)
    if cls_origin is Annotated:
        return generic_issubclass(scls, cls_args[0])

    if scls_origin.__class__ is _TypedDictMeta:
        if cls_origin.__class__ is _TypedDictMeta:
            return all(k in cls_origin.__annotations__ for k in scls_origin.__annotations__)
        return generic_issubclass(cls_origin, dict)
    if cls_origin.__class__ is _TypedDictMeta:
        return generic_issubclass(scls, dict)

    if origin_is_union(scls_origin):
        return all(map(generic_issubclass, scls_args, repeat(cls)))
    if origin_is_union(cls_origin):
        return generic_issubclass(scls, cls_args)

    if origin_is_literal(scls_origin) and origin_is_literal(cls_origin):
        return set(scls_args) <= set(cls_args)
    if isinstance(cls, TypeVar):
        if cls.__constraints__:
            return any(generic_issubclass(scls, p) for p in cls.__constraints__)
        return generic_issubclass(scls, cls.__bound__) if cls.__bound__ else True
    try:
        if not issubclass(scls_origin, cls_origin):
            return False
    except TypeError:
        return False

    if not cls_args:
        return True

    return len(scls_args) == len(cls_args) and all(map(generic_issubclass, scls_args, cls_args))
